"""Clean Jupyter notebooks of execution counts, metadata, and outputs."""

import argparse
import pathlib
import subprocess
import sys
from typing import NoReturn

import nbformat

VERSION = "1.4.3"
ATTRIBUTE = "*.ipynb filter=nb-clean"


def error(message: str, code: int) -> NoReturn:
    """Print error message to stderr and exit with code.

    Parameters
    ----------
    message : str
        Error message, printed to stderr.
    code : int
        Return code.

    """
    print(f"nb-clean: error: {message}", file=sys.stderr)
    sys.exit(code)


def git(*args: str) -> str:
    """Call a Git subcommand.

    Parameters
    ----------
    *args : str
        Git subcommand and arguments.

    Returns
    -------
    str
        stdout from Git.

    Examples
    --------
    >>> git('rev-parse', '--git-dir')
    .git

    """
    process = subprocess.run(
        ["git"] + list(args),
        stderr=subprocess.PIPE,
        stdout=subprocess.PIPE,
        check=False,
    )

    if process.returncode:
        error(process.stderr.decode(), process.returncode)

    stdout: str = process.stdout.decode().strip()
    return stdout


def attributes_path() -> pathlib.Path:
    """Get the attributes file path for the current Git repository.

    Returns
    -------
    pathlib.Path
        Path to the Git attributes file.

    Examples
    --------
    >>> attributes_path()
    PosixPath('.git/info/attributes')

    """
    git_dir = git("rev-parse", "--git-dir")
    return pathlib.Path(git_dir) / "info" / "attributes"


def print_version(args: argparse.Namespace) -> None:
    """Print the version number.

    Parameters
    ----------
    args : argparse.Namespace
        Unused.

    """
    del args  # Unused.
    print(f"nb-clean {VERSION}")


def configure_git(args: argparse.Namespace) -> None:
    """Configure Git repository to use nb-clean filter.

    Parameters
    ----------
    args : argparse.Namespace
        Unused.

    """
    del args  # Unused.

    git("config", "filter.nb-clean.clean", "nb-clean clean")

    attributes = attributes_path()

    if attributes.is_file() and ATTRIBUTE in attributes.read_text():
        return

    with attributes.open("a") as file:
        file.write(f"\n{ATTRIBUTE}\n")


def unconfigure_git(args: argparse.Namespace) -> None:
    """Remove nb-clean filter from the Git repository.

    Parameters
    ----------
    args : argparse.Namespace
        Unused.

    """
    del args  # Unused.

    attributes = attributes_path()

    if attributes.is_file():
        original_contents = attributes.read_text().split("\n")
        revised_contents = [
            line for line in original_contents if line != ATTRIBUTE
        ]
        attributes.write_text("\n".join(revised_contents))

    git("config", "--remove-section", "filter.nb-clean")


def check(args: argparse.Namespace) -> None:
    """Check a notebook is clean of execution counts, metadata, and outputs.

    Parameters
    ----------
    args : argparse.Namespace
        Arguments parsed from the command line.

    """
    notebook = nbformat.read(args.input, as_version=nbformat.NO_CONVERT)
    dirty = False

    for index, cell in enumerate(notebook.cells):
        prefix = f"{args.input.name} cell {index}"

        if cell["metadata"]:
            print(f"{prefix}: metadata")
            dirty = True
        if cell["cell_type"] == "code":
            if cell["execution_count"]:
                print(f"{prefix}: execution count")
                dirty = True
            if cell["outputs"]:
                print(f"{prefix}: outputs")
                dirty = True

    if dirty:
        sys.exit(1)


def clean(args: argparse.Namespace) -> None:
    """Clean notebook of execution counts, metadata, and outputs.

    Parameters
    ----------
    args : argparse.Namespace
        Arguments parsed from the command line.

    """
    notebook = nbformat.read(args.input, as_version=nbformat.NO_CONVERT)

    for cell in notebook.cells:
        cell["metadata"] = {}
        if cell["cell_type"] == "code":
            cell["execution_count"] = None
            cell["outputs"] = []

    nbformat.write(notebook, args.output)


def main() -> None:
    """Parse command line arguments and call corresponding function."""
    parser = argparse.ArgumentParser(allow_abbrev=False, description=__doc__)
    subparsers = parser.add_subparsers(dest="subcommand")
    subparsers.required = True

    version_parser = subparsers.add_parser(
        "version", help="print version number"
    )
    version_parser.set_defaults(func=print_version)

    configure_parser = subparsers.add_parser(
        "configure-git",
        help="configure Git filter to clean notebooks before staging",
    )
    configure_parser.set_defaults(func=configure_git)

    unconfigure_parser = subparsers.add_parser(
        "unconfigure-git",
        help="remove Git filter that cleans notebooks before staging",
    )
    unconfigure_parser.set_defaults(func=unconfigure_git)

    check_parser = subparsers.add_parser(
        "check",
        help=(
            "check a notebook is clean of cell execution counts, metadata,"
            " and outputs"
        ),
    )
    check_parser.add_argument(
        "-i",
        "--input",
        nargs="?",
        type=argparse.FileType("r"),
        default=sys.stdin,
        help="input file",
    )
    check_parser.set_defaults(func=check)

    clean_parser = subparsers.add_parser(
        "clean",
        help="clean notebook of cell execution counts, metadata, and outputs",
    )
    clean_parser.add_argument(
        "-i",
        "--input",
        nargs="?",
        type=argparse.FileType("r"),
        default=sys.stdin,
        help="input file",
    )
    clean_parser.add_argument(
        "-o",
        "--output",
        nargs="?",
        type=argparse.FileType("w"),
        default=sys.stdout,
        help="output file",
    )
    clean_parser.set_defaults(func=clean)

    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

modules = \
['nb_clean']
install_requires = \
['nbformat>=4.4']

entry_points = \
{'console_scripts': ['nb-clean = nb_clean:main']}

setup_kwargs = {
    'name': 'nb-clean',
    'version': '1.4.3',
    'description': 'Clean Jupyter notebooks for versioning',
    'long_description': '# nb-clean\n\n`nb-clean` cleans Jupyter notebooks of cell execution counts, metadata, and\noutputs, preparing them for committing to version control. It provides a Git\nfilter to automatically clean notebooks before they are staged, and can also be\nused as a standalone tool outside Git or with other version control systems. It\ncan determine if a notebook is clean or not, which can be used as a check in\nyour continuous integration pipelines.\n\n## Installation\n\nTo install the latest release from [PyPI], use [pip]:\n\n```bash\npython -m pip install nb-clean\n```\n\nAlternately, in Python projects using [Poetry] or [Pipenv] for dependency\nmanagement, add `nb-clean` as a development dependency with\n`poetry add --dev nb-clean` or `pipenv install --dev nb-clean`. `nb-clean`\nrequires Python 3.6 or later.\n\n## Usage\n\n### Cleaning\n\nTo install a filter in an existing Git repository to automatically clean\nnotebooks before they are staged, run the following from the working tree:\n\n```bash\nnb-clean configure-git\n```\n\n`nb-clean` will configure a filter in the Git repository in which it is run, and\nwill not mutate your global or system Git configuration. To remove the filter,\nrun:\n\n```bash\nnb-clean unconfigure-git\n```\n\nAside from usage from a filter in a Git repository, you can also clean up a\nJupyter notebook manually with:\n\n```bash\nnb-clean clean -i original.ipynb -o cleaned.ipynb\n```\n\nor by passing the notebook contents on stdin:\n\n```bash\nnb-clean clean < original.ipynb > cleaned.ipynb\n```\n\n### Checking\n\nYou can check if a notebook is clean with:\n\n```bash\nnb-clean check -i notebook.ipynb\n```\n\nor by passing the notebook contents on stdin:\n\n```bash\nnb-clean check < notebook.ipynb\n```\n\n`nb-clean` will exit with status code 0 if the notebook is clean, and status\ncode 1 if it is not. `nb-clean` will also print details of cell execution\ncounts, metadata, and outputs it finds.\n\n## Copyright\n\nCopyright © 2017-2020 [Scott Stevenson].\n\n`nb-clean` is distributed under the terms of the [ISC licence].\n\n[isc licence]: https://opensource.org/licenses/ISC\n[pip]: https://pip.pypa.io/en/stable/\n[pipenv]: https://pipenv.readthedocs.io/en/latest/\n[poetry]: https://python-poetry.org/\n[pypi]: https://pypi.org/project/nb-clean/\n[scott stevenson]: https://scott.stevenson.io\n',
    'author': 'Scott Stevenson',
    'author_email': 'scott@stevenson.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/srstevenson/nb-clean',
    'package_dir': package_dir,
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
